////////////////////////////////////////////////////////////////////////////
// NoteCase notes manager project <http://notecase.sf.net>
//
// This code is licensed under BSD license.See "license.txt" for more details.
//
// File: Implements window for file export options
////////////////////////////////////////////////////////////////////////////

#include "FileExportDlg.h"
#include "gui/FileDialog.h"
#include "lib/IniFile.h"
#include "support.h"
#include <string.h>
#include <string>
#include <algorithm>
#include "mru.h"

extern GtkWidget *window1;
static void on_pick_file_clicked(GtkButton *button, gpointer user_data);
static void on_save_clicked(GtkButton *button, gpointer user_data);
static void on_format_combo_selected (GtkComboBox *widget, gpointer user_data);
int gtkMessageBox(const char *szText, int nButtons = GTK_BUTTONS_OK, int nIcon = GTK_MESSAGE_INFO);
const char *GetIniFile();

FileExportDlg::FileExportDlg()
{
	// init formats table
	m_formats[0].szName   = _("NoteCase document (*.ncd)");
	m_formats[0].szFilter = "*.ncd";
	m_formats[1].szName   = _("NoteCase encrypted document (*.nce)");
	m_formats[1].szFilter = "*.nce";
	m_formats[2].szName   = _("NoteCenter document (*.hnc)");
	m_formats[2].szFilter = "*.hnc";
	m_formats[3].szName   = _("HTML file (*.html)");
	m_formats[3].szFilter = "*.html";
	m_formats[4].szName   = _("Text file (*.txt)");
	m_formats[4].szFilter = "*.txt";

	Create();
}

FileExportDlg::~FileExportDlg()
{
	Destroy();
	gtk_window_present(GTK_WINDOW(window1)); //activate main window
}

void FileExportDlg::Create()
{
	m_pDialog = create_file_export_dialog (window1);

	// fill export sources
	GtkWidget *combobox1 = lookup_widget(m_pDialog, "combobox1");
	gtk_combo_box_append_text (GTK_COMBO_BOX (combobox1), _("Selected branch"));
	gtk_combo_box_append_text (GTK_COMBO_BOX (combobox1), _("Selected node only"));
	gtk_combo_box_append_text (GTK_COMBO_BOX (combobox1), _("Entire document"));

	// fill format combo box
	GtkWidget *combobox2 = lookup_widget(m_pDialog, "combobox2");
	for(unsigned int i=0; i<sizeof(m_formats)/sizeof(m_formats[0]); i++)
		gtk_combo_box_append_text (GTK_COMBO_BOX (combobox2), m_formats[i].szName);

	IniFile file;
	file.Load(GetIniFile());

	//remember last used export source in INI
	int nExportSource = 0;
	file.GetValue("Export", "DocumentSource", nExportSource, 0);
	if(0 > nExportSource || nExportSource > 2)
		nExportSource = 0;

	gtk_combo_box_set_active (GTK_COMBO_BOX (combobox1), nExportSource);

	//remember last used export format in INI
	int nExportFormat = 0;
	file.GetValue("Export", "TargetFormat", nExportFormat, 0);
	if(0 > nExportFormat || nExportFormat > 4)
		nExportFormat = 0;

	gtk_combo_box_set_active (GTK_COMBO_BOX (combobox2), nExportFormat);
}

void FileExportDlg::ForceFormatExtension()
{
	std::string strFile = GetExportFile();
	if(!strFile.empty())
	{
		//get export format
		GtkWidget *combobox2 = lookup_widget(m_pDialog, "combobox2");
		
		int nPos = gtk_combo_box_get_active (GTK_COMBO_BOX (combobox2));
		if(nPos >= 0)
		{
			std::string strExt = m_formats[nPos].szFilter;
			strExt.erase(strExt.begin(), strExt.begin()+1);

			unsigned int nSize = strExt.size();

			// make sure file is terminated with the matchign extension
			if( strFile.size() > nSize &&
			    strFile.substr(strFile.size()-nSize) == strExt)
				return;

			//strip previous extension TOFIX?
			std::string::size_type nExt = strFile.rfind('.');
			if(nExt != std::string::npos)
				strFile = strFile.substr(0, nExt);

			//append extension
			strFile += strExt;

			//set new name
			GtkWidget *entry5 = lookup_widget(m_pDialog, "entry5");
			gtk_entry_set_text(GTK_ENTRY(entry5), strFile.c_str());
		}
	}
}

int FileExportDlg::GetExportMode()
{
	GtkWidget *combobox1 = lookup_widget(m_pDialog, "combobox1");
	return 	gtk_combo_box_get_active (GTK_COMBO_BOX (combobox1));
}

const char *FileExportDlg::GetExportFile()
{
	GtkWidget *entry5 = lookup_widget(m_pDialog, "entry5");
	return 	gtk_entry_get_text(GTK_ENTRY(entry5));
}

GtkWidget* FileExportDlg::create_file_export_dialog (GtkWidget* parent)
{
	GtkWidget *file_export_dialog;
	GtkWidget *dialog_vbox6;
	GtkWidget *table7;
	GtkWidget *label24;
	GtkWidget *label25;
	GtkWidget *combobox1;
	GtkWidget *combobox2;
	GtkWidget *label26;
	GtkWidget *hbox3;
	GtkWidget *entry5;
	GtkWidget *button5;
	GtkWidget *image2;
	GtkWidget *dialog_action_area6;
	GtkWidget *cancelbutton6;
	GtkWidget *okbutton6;
	
	file_export_dialog = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (file_export_dialog), _("Export"));
	gtk_window_set_modal (GTK_WINDOW (file_export_dialog), TRUE);
	gtk_window_set_destroy_with_parent (GTK_WINDOW (file_export_dialog), TRUE);
#ifndef _WIN32  //TOFIX API is buggy on Win32 (kills modal dialog state)
	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (file_export_dialog), TRUE);
#endif
	gtk_window_set_skip_pager_hint (GTK_WINDOW (file_export_dialog), TRUE);
	gtk_window_set_type_hint (GTK_WINDOW (file_export_dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
	if(parent)
		gtk_window_set_transient_for(GTK_WINDOW (file_export_dialog), GTK_WINDOW(parent));   //set parent
#if GTK_CHECK_VERSION(2,4,0) //new API TOFIX set proper version
	#ifndef _WIN32  //TOFIX API is buggy on Win32 (kills modal dialog state)
	  gtk_window_set_keep_above(GTK_WINDOW (file_export_dialog), TRUE);
	#endif
#endif

	dialog_vbox6 = GTK_DIALOG (file_export_dialog)->vbox;
	gtk_widget_show (dialog_vbox6);
	
	table7 = gtk_table_new (3, 2, FALSE);
	gtk_widget_show (table7);
	gtk_box_pack_start (GTK_BOX (dialog_vbox6), table7, TRUE, TRUE, 0);
	
	label24 = gtk_label_new (_("Document source:"));
	gtk_widget_show (label24);
	gtk_table_attach (GTK_TABLE (table7), label24, 0, 1, 0, 1,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label24), 0, 0.5);
	
	label25 = gtk_label_new (_("File format:"));
	gtk_widget_show (label25);
	gtk_table_attach (GTK_TABLE (table7), label25, 0, 1, 1, 2,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label25), 0, 0.5);
	
	combobox1 = gtk_combo_box_new_text ();
	gtk_widget_show (combobox1);
	gtk_table_attach (GTK_TABLE (table7), combobox1, 1, 2, 0, 1,
		(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		(GtkAttachOptions) (GTK_FILL), 0, 0);
	
	combobox2 = gtk_combo_box_new_text ();
	gtk_widget_show (combobox2);
	gtk_table_attach (GTK_TABLE (table7), combobox2, 1, 2, 1, 2,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (GTK_FILL), 0, 0);
	
	label26 = gtk_label_new (_("File name:"));
	gtk_widget_show (label26);
	gtk_table_attach (GTK_TABLE (table7), label26, 0, 1, 2, 3,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label26), 0, 0.5);
	
	hbox3 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox3);
	gtk_table_attach (GTK_TABLE (table7), hbox3, 1, 2, 2, 3,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (GTK_FILL), 0, 0);
	
	entry5 = gtk_entry_new ();
	gtk_widget_show (entry5);
	gtk_box_pack_start (GTK_BOX (hbox3), entry5, TRUE, TRUE, 0);
	
	button5 = gtk_button_new ();
	gtk_widget_show (button5);
	gtk_box_pack_start (GTK_BOX (hbox3), button5, FALSE, FALSE, 0);
	
	image2 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show (image2);
	gtk_container_add (GTK_CONTAINER (button5), image2);
	
	dialog_action_area6 = GTK_DIALOG (file_export_dialog)->action_area;
	gtk_widget_show (dialog_action_area6);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area6), GTK_BUTTONBOX_END);
	
	cancelbutton6 = gtk_button_new_from_stock ("gtk-cancel");
	gtk_widget_show (cancelbutton6);
	gtk_dialog_add_action_widget (GTK_DIALOG (file_export_dialog), cancelbutton6, GTK_RESPONSE_CANCEL);
	GTK_WIDGET_SET_FLAGS (cancelbutton6, GTK_CAN_DEFAULT);
	
	okbutton6 = gtk_button_new_with_mnemonic (_("Save"));
	gtk_widget_show (okbutton6);
	gtk_container_add (GTK_CONTAINER(dialog_action_area6), okbutton6);
	GTK_WIDGET_SET_FLAGS (okbutton6, GTK_CAN_DEFAULT);
	
	g_signal_connect(button5,   "clicked", G_CALLBACK (on_pick_file_clicked), this);
	g_signal_connect(okbutton6, "clicked", G_CALLBACK (on_save_clicked), this);
	g_signal_connect(combobox2, "changed", G_CALLBACK (on_format_combo_selected), this);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (file_export_dialog, file_export_dialog, "file_export_dialog");
	GLADE_HOOKUP_OBJECT_NO_REF (file_export_dialog, dialog_vbox6, "dialog_vbox6");
	GLADE_HOOKUP_OBJECT (file_export_dialog, table7, "table7");
	GLADE_HOOKUP_OBJECT (file_export_dialog, label24, "label24");
	GLADE_HOOKUP_OBJECT (file_export_dialog, label25, "label25");
	GLADE_HOOKUP_OBJECT (file_export_dialog, combobox1, "combobox1");
	GLADE_HOOKUP_OBJECT (file_export_dialog, combobox2, "combobox2");
	GLADE_HOOKUP_OBJECT (file_export_dialog, label26, "label26");
	GLADE_HOOKUP_OBJECT (file_export_dialog, hbox3, "hbox3");
	GLADE_HOOKUP_OBJECT (file_export_dialog, entry5, "entry5");
	GLADE_HOOKUP_OBJECT (file_export_dialog, button5, "button5");
	GLADE_HOOKUP_OBJECT (file_export_dialog, image2, "image2");
	GLADE_HOOKUP_OBJECT_NO_REF (file_export_dialog, dialog_action_area6, "dialog_action_area6");
	GLADE_HOOKUP_OBJECT (file_export_dialog, cancelbutton6, "cancelbutton6");
	GLADE_HOOKUP_OBJECT (file_export_dialog, okbutton6, "okbutton6");
	
	gtk_widget_grab_focus (entry5);
	gtk_widget_grab_default (okbutton6);

	return file_export_dialog;
}

void on_pick_file_clicked (GtkButton *button, gpointer user_data)
{
	FileExportDlg *pDlg = (FileExportDlg *)user_data;
	GtkWidget *combobox2 = lookup_widget(pDlg->m_pDialog, "combobox2");
	GtkWidget *entry5    = lookup_widget(pDlg->m_pDialog, "entry5");

	int nFmt = gtk_combo_box_get_active (GTK_COMBO_BOX (combobox2));

	//TOFIX set initial directory from INI (store last used)
	FileDialog dlg(FALSE, _("Select export file"));

	//set single format filter based on current format selection
	if(nFmt >= 0)
		dlg.AddFilter(pDlg->m_formats[nFmt].szName, pDlg->m_formats[nFmt].szFilter);

	// pick file name and write it to text field
	if(dlg.DoModal())
	{
		gtk_entry_set_text(GTK_ENTRY(entry5), dlg.GetFilename());
		pDlg->ForceFormatExtension();
	}
}

void on_save_clicked (GtkButton *button, gpointer user_data)
{
	FileExportDlg *pDlg = (FileExportDlg *)user_data;

	const char *szFile = pDlg->GetExportFile();
	if(NULL == szFile || 0 == strlen(szFile))
	{
		gtkMessageBox(_("You must define the export file name!"));
		return;
	}
	
	pDlg->ForceFormatExtension();

	// remeber state in INI
	GtkWidget *combobox1 = lookup_widget(pDlg->m_pDialog, "combobox1");
	GtkWidget *combobox2 = lookup_widget(pDlg->m_pDialog, "combobox2");

	IniFile file;
	file.Load(GetIniFile());
	int nExportSource = gtk_combo_box_get_active (GTK_COMBO_BOX (combobox1));
	file.SetValue("Export", "DocumentSource", nExportSource);
	int nExportFormat = gtk_combo_box_get_active (GTK_COMBO_BOX (combobox2));
	file.SetValue("Export", "TargetFormat", nExportFormat);
	file.Save();

	//terminate with OK
	gtk_dialog_response(GTK_DIALOG(pDlg->m_pDialog), GTK_RESPONSE_OK);
}

void on_format_combo_selected (GtkComboBox *widget, gpointer user_data)
{
	FileExportDlg *pDlg = (FileExportDlg *)user_data;
	pDlg->ForceFormatExtension();
}

