////////////////////////////////////////////////////////////////////////////
// NoteCase notes manager project <http://notecase.sf.net>
//
// This code is licensed under BSD license.See "license.txt" for more details.
//
// File: Implements layer for encrypting/decrypting the data using Blowfish
////////////////////////////////////////////////////////////////////////////

#include "IOLayerEnc.h"
#include "string.h"
#include "debug.h"

#ifndef min
 #define min(x,y) (((x)<(y))?(x):(y))
#endif

IOLayerEnc::IOLayerEnc()
{
	m_nDataSize = 0;
	m_nOutSize = 0;
}

IOLayerEnc::~IOLayerEnc()
{
}

int IOLayerEnc::Process(char *szBuffer, int nLen)
{
	int nBytes = 0;
	if(MODE_WRITE == m_nMode)
		nBytes = Write(szBuffer, nLen);
	else
		nBytes = Read(szBuffer, nLen);

	//store output size for this operation
	m_nLastOutBytes = nBytes;

	//NOTE: next layer is called within Read/Write
	return nBytes;
}

void IOLayerEnc::SetPassword(const char *szPass)
{
	m_crypt.Initialize((BYTE *)szPass, strlen(szPass));
}

int IOLayerEnc::Read(char *szBuffer, int nLen)
{
	TRACE("IOLayerEnc::Read (content size=%d)\n", nLen);

	int nRead = nLen;
	m_crypt.Decode((BYTE *)szBuffer, (BYTE *)szBuffer, nRead); //inplace decode
	//TRACE("IOLayerEnc::Read decoded=\n%s\n", szBuffer);

	szBuffer[nRead] = '\0';
	m_nOutSize = nRead;
	
	m_nProcessedInBytes += nLen;

	//call next layer attached?
	if(NULL != m_pNextLayer && nRead > 0){
		nRead = m_pNextLayer->Process(szBuffer, nRead);
	}

	return nRead;
}

int IOLayerEnc::Write(char *szBuffer, int nLen)
{
	TRACE("IOLayerEnc::Write (content size=%d)\n", nLen);

	int nPos = 0;
	int nTotalSize = 0;
	int nTotalSizeExt = 0;
	int nAvailSpace = ENC_BUFFER_SIZE-m_nDataSize;

	m_nProcessedInBytes += nLen;

	while(nLen > 0)
	{
		//prepare new buffer (append new data to its end)
		int nAdd = min(nLen, nAvailSpace);
		memcpy(m_szBuffer+m_nDataSize, szBuffer+nPos, nAdd);

		//refresh variables
		m_nDataSize += nAdd;
		nLen -= nAdd;
		nPos += nAdd;
		nAvailSpace = ENC_BUFFER_SIZE-m_nDataSize;

		//flush data if buffer full
		if(0 == nAvailSpace){
			int nBytes = WriteBuffer();
			nTotalSize += nBytes;

			//call next layer attached?
			if(NULL != m_pNextLayer && nBytes > 0){
				nTotalSizeExt += m_pNextLayer->Process(m_szBuffer, nBytes);
			}
		}
	}

	m_nOutSize = nTotalSize;
	return nTotalSizeExt;
}

int IOLayerEnc::Finalize()
{
	//write anything left in the buffer
	if(MODE_WRITE == m_nMode)
	{
		TRACE("IOLayerEnc::Finalize - size to be flushed =%d\n",m_nDataSize);

		int nBytes = WriteBuffer();
		
		//call next layer attached?
		if(NULL != m_pNextLayer && nBytes > 0){
			m_pNextLayer->Process(m_szBuffer, nBytes);	//TOFIX
			return m_pNextLayer->Finalize();
		}

		return nBytes;
	}
	return 0;
}

int IOLayerEnc::WriteBuffer()
{
	//encode and write buffer into the file
	if(m_nDataSize > 0)
	{
		int nOrigSize = m_nDataSize;
		int nEncSize  = m_crypt.Encode((BYTE *)m_szBuffer, (BYTE *)m_szBuffer, nOrigSize);
		TRACE("IOLayerEnc::WriteBuffer in size=%d, out size=%d\n", nOrigSize, nEncSize);

		m_nDataSize = 0;
		return nEncSize;
	}
	return 0;
}
